local VERSIONMAJOR = 2.1

ShivvAudio = LibStub('AceAddon-3.0'):NewAddon('ShivvAudio', 'AceConsole-3.0', 'AceEvent-3.0')
ShivvAudio.internalMusicRef = {
	shivvInternalAuction = 0,
	shivvInternalBank = 0,
	shivvInternalShop = 0,
	shivvInternalLooting = 0,
	shivvInternalCrafting = 0,
	shivvInternalCombatOutside1 = 0,
	shivvInternalCombatOutside2 = 0,
	shivvInternalCombatOutside3 = 0,
	shivvInternalCombatOutsideElite = 0,
	shivvInternalCombatParty1 = 0,
	shivvInternalCombatParty2 = 0,
	shivvInternalCombatParty3 = 0,
	shivvInternalCombatPartyBoss = 0,
	shivvInternalCombatBattleground = 0,
	shivvInternalCombatRaidTrash = 0,
	shivvInternalCombatRaidBoss = 0,
}

local L = LibStub('AceLocale-3.0'):GetLocale('ShivvAudio')

local defaults = {
	profile = {
		-- global
		enableAudibleAuras = false,
		enableMelodicVitals = false,
		enableMusicalEvents = false,
		enableSymphoniousCombat = false,
		gameMusicControl = false,
		-- audible auras
		audibleAuras = {},
		-- melodic vitals
		melodicVitals = {
			healthDB = {},
			powerDB = {},
			repeatTimer = 5
		},
		-- musical events
		musicalEvents = {
			achievement = 0, -- sound
			auction = 0, -- music
			bank = 0, -- music
			crafting = 0, -- music
			death = 0, -- sound
			levelup = 0, -- sound
			lootclose = 0, -- sound
			looting = 0, -- music
			lootopen = 0, -- sound
			shop = 0 -- music
		},
		-- symphonious combat
		symphoniousCombat = {
			advanced = false,
			enableOutside = false,
			outsideMaxT1 = 1,
			outsideMaxT2 = 2,
			enableBattleground = false,
			enableParty = false,
			partyMaxT1 = 1,
			partyMaxT2 = 3,
			enableRaid = false,
			audio = {
				-- music entries
				outside1 = 0,
				outside2 = 0,
				outside3 = 0,
				outsideelite = 0,
				party1 = 0,
				party2 = 0,
				party3 = 0,
				partyboss = 0,
				battleground = 0,
				raidtrash = 0,
				raidboss = 0,
				-- sound entries
				outsideenter = 0,
				outsideleave = 0,
				partyenter = 0,
				partyleave = 0,
				battlegroundenter = 0,
				battlegroundleave = 0,
				raidenter = 0,
				raidleave = 0
			}
		}
	},
	global = {
		mediaDB = {}
	}
}

-- ---------------------------- --
-- ON ZONE DELAYED ACTION TIMER --
-- ---------------------------- --
do
	local handler = CreateFrame('Frame')
	local timer

	local function DelayedActionTimer(frame, elapsed)
		timer = timer - elapsed

		if (timer <= 0) then
			handler:SetScript('OnUpdate', nil)
			ShivvAudio:DelayedOnZone()
		end
	end

	function ShivvAudio:StartDelayedZoneTimer(time)
		timer = time
		handler:SetScript('OnUpdate', DelayedActionTimer)
	end
end

function ShivvAudio:DelayedOnZone() -- fires a short while after a zone to setup things that aren't immediately available
	self:MusicForcePlay() -- force play any supposed-to-be active music
end

-- -------- --
-- MOD INIT --
-- -------- --
function ShivvAudio:OnInitialize()
	self.db = LibStub('AceDB-3.0'):New('ShivvAudioDB', defaults)
	self:RegisterChatCommand('shivv', 'ChatHandler')
	self:RegisterEvent('PLAYER_ENTERING_WORLD')

	-- update handling
	if (not self.db.profile.versionMajor) then -- old code pre update
		self:Print('Updating Database to latest version...')

		if (self.db.profile.auraEnableTracking) then
			self.db.profile.auraEnableTracking = nil
			self.db.profile.enableAudibleAuras = true
		end

		-- couldn't convert to new system, wiping it
		if (self.db.profile.vitalEnableHealthTracking) then
			self.db.profile.vitalEnableHealthTracking = nil
		end

		if (self.db.profile.vitalEnablePowerTracking) then
			self.db.profile.vitalEnablePowerTracking = nil
		end

		if (self.db.profile.vitalHealthDB) then
			self.db.profile.vitalHealthDB = nil
		end

		if (self.db.profile.vitalPowerDB) then
			self.db.profile.vitalPowerDB = nil
		end

		if (self.db.profile.vitalRepeatTimer) then
			self.db.profile.melodicVitals.repeatTimer = self.db.profile.vitalRepeatTimer
			self.db.profile.vitalRepeatTimer = nil
		end

		if (self.db.profile.auraDB) then
			self.db.profile.audibleAuras = self.db.profile.auraDB
			self.db.profile.auraDB = nil
		end

		if (self.db.profile.eventDB) then
			self.db.profile.eventDB = nil
		end

		self.db.profile.versionMajor = VERSIONMAJOR -- new version
	elseif (self.db.profile.versionMajor < VERSIONMAJOR) then -- code pre latest update
		-- nothing yet
	end
end

function ShivvAudio:OnEnable()
	self:PopulateShivvInternalMusicRef()
end

function ShivvAudio:PLAYER_ENTERING_WORLD()
	self:StartDelayedZoneTimer(1.0) -- delay by a second after zoning
end

-- ------------- --
-- SOUND HANDLER --
-- ------------- --
function ShivvAudio:SoundHandler(sound)
	if (sound and sound ~= 0) then
		PlaySoundFile(self.db.global.mediaDB[sound].path)
	end
end

-- ------------- --
-- MUSIC HANDLER --
-- ------------- --
do
	local activeMusic = {}

	function ShivvAudio:CleanupMusic()
		-- clear out and kill any active music
		while (#(activeMusic) > 0) do
			tremove(activeMusic, 1)
		end

		self:MusicStop()
	end

	function ShivvAudio:MusicForcePlay()
		-- force play the #1 music (if any)
		if (#(activeMusic) >= 1) then
			self:MusicStart(activeMusic[1])
		end
	end

	function ShivvAudio:IsMusicActive()
		if (#(activeMusic) > 0) then
			return true
		else
			return false
		end
	end

	function ShivvAudio:MusicMultiKill(mtk) -- table of auras to kill
		if (#(activeMusic) > 1) then -- 2 or more entries
			for i = #activeMusic, 2, -1 do -- go through non-active entries backwards since we're removing entries
				if (mtk[activeMusic[i]]) then -- item exists, remove
					tremove(activeMusic, i)
				end
			end
		end

		-- now non actives are gone, see if the active is in the list
		if (mtk[activeMusic[1]]) then -- it is, stop music the usual way
			tremove(activeMusic, 1)
			self:MusicStop()
		end
	end

	function ShivvAudio:MusicStart(aura)
		if (self.db.profile.gameMusicControl and GetCVar('Sound_EnableMusic') == '0') then -- we control music and its off, turn it on
			SetCVar('Sound_EnableMusic', '1')
		end

		if (self.internalMusicRef[aura]) then -- this is an internally activated music file
			PlayMusic(self.db.global.mediaDB[self.internalMusicRef[aura]].path)
		else
			PlayMusic(self.db.global.mediaDB[self.db.profile.audibleAuras.player[aura].music].path)
		end
	end

	function ShivvAudio:MusicStop()
		StopMusic()

		if (self.db.profile.gameMusicControl and GetCVar('Sound_EnableMusic') == '1') then -- we control music and its on, kill it
			SetCVar('Sound_EnableMusic', '0')
		end
	end

	function ShivvAudio:MusicHandler(aura, active)
		if (active) then -- new music to activate, do so and add to tracking table if there is one
			if (self.internalMusicRef[aura]) then
				if (self.internalMusicRef[aura] == 0) then
					return
				end
			elseif (not self.db.profile.audibleAuras.player) then
				self:Print('Music Handler - This should not happen, aura: ' .. aura)
				return
			elseif (self.db.profile.audibleAuras.player[aura].music == 0) then
				return
			end

			tinsert(activeMusic, 1, aura)

			self:MusicStart(aura)
		else -- music to stop plauying, do so, remove from tracking table and look for replacement (if any)
			for index, active in ipairs(activeMusic) do
				if (aura == active) then -- found this aura in active data
					if (index ~= 1) then -- this wasn't currently playing, just remove entry
						tremove(activeMusic, index)
					else -- this is currently playing, remove entry and look for replacement
						tremove(activeMusic, index)

						if (#(activeMusic) >= 1) then -- there is another entry, play that instead
							self:MusicStart(activeMusic[1])
						else -- no other entry, stop music
							self:MusicStop()
						end
					end

					break -- in any case, we're done here
				end
			end
		end
	end
end

-- ---- --
-- MISC --
-- ---- --
function ShivvAudio:PopulateShivvInternalMusicRef()
	-- populate from options for internal calls
	local me = self.db.profile.musicalEvents

	self.internalMusicRef.shivvInternalAuction = me.auction
	self.internalMusicRef.shivvInternalBank = me.bank
	self.internalMusicRef.shivvInternalShop = me.shop
	self.internalMusicRef.shivvInternalLooting = me.looting
	self.internalMusicRef.shivvInternalCrafting = me.crafting

	local sc = self.db.profile.symphoniousCombat.audio

	self.internalMusicRef.shivvInternalCombatOutside1 = sc.outside1
	self.internalMusicRef.shivvInternalCombatOutside2 = sc.outside2
	self.internalMusicRef.shivvInternalCombatOutside3 = sc.outside3
	self.internalMusicRef.shivvInternalCombatOutsideElite = sc.outsideelite
	self.internalMusicRef.shivvInternalCombatParty1 = sc.party1
	self.internalMusicRef.shivvInternalCombatParty2 = sc.party2
	self.internalMusicRef.shivvInternalCombatParty3 = sc.party3
	self.internalMusicRef.shivvInternalCombatPartyBoss = sc.partyboss
	self.internalMusicRef.shivvInternalCombatBattleground = sc.battleground
	self.internalMusicRef.shivvInternalCombatRaidTrash = sc.raidtrash
	self.internalMusicRef.shivvInternalCombatRaidBoss = sc.raidboss
end

function ShivvAudio:ChatHandler(input)
	if (input == '') then
		self:BuildGUI() -- open gui if no commands
	elseif (input == 'kill') then
		self:CleanupMusic()
	else
		self:Print(L.ChatHelp)
	end
end

function ShivvAudio:RemoveMedia(media)
	local changed = 0

	self:Print(L.RemovingMedia)

	for char, charData in pairs(ShivvAudioDB.profiles) do
		-- first: audible auras
		if (charData.audibleAuras and charData.audibleAuras) then
			for unit in pairs(charData.audibleAuras) do
				for buff in pairs(charData.audibleAuras[unit]) do
					if (charData.audibleAuras[unit][buff].onGain == media) then
						charData.audibleAuras[unit][buff].onGain = 0
						changed = changed + 1
					end
					if (charData.audibleAuras[unit][buff].onLoss == media) then
						charData.audibleAuras[unit][buff].onLoss = 0
						changed = changed + 1
					end
					if (charData.audibleAuras[unit][buff].music == media) then
						charData.audibleAuras[unit][buff].music = 0
						changed = changed + 1
					end
				end
			end
		end

		-- then melodic vitals
		if (charData.melodicVitals) then
			if (charData.melodicVitals.healthDB) then
				for unit in pairs(charData.melodicVitals.healthDB) do
					if (charData.melodicVitals.healthDB[unit].sound == media) then
						charData.melodicVitals.healthDB[unit].sound = 0
						changed = changed + 1
					end
				end
			end

			if (charData.melodicVitals.powerDB) then
				for unit in pairs(charData.melodicVitals.powerDB) do
					if (charData.melodicVitals.powerDB[unit].sound == media) then
						charData.melodicVitals.powerDB[unit].sound = 0
						changed = changed + 1
					end
				end
			end
		end

		-- then musical events
		if (charData.musicalEvents) then
			for event in pairs(charData.musicalEvents) do
				if (charData.musicalEvents[event] == media) then
					charData.musicalEvents[event] = 0
					changed = changed + 1
				end
			end
		end

		-- and finally symphonious combat
		if (charData.symphoniousCombat and charData.symphoniousCombat.audio) then
			for a in pairs(charData.symphoniousCombat.audio) do
				if (charData.symphoniousCombat.audio[a] == media) then
					charData.symphoniousCombat.audio[a] = 0
					changed = changed + 1
				end
			end
		end

		self:Print(format(L.RemovingMediaForChar, changed, char))
	end

	self:GetModule('MelodicVitals'):OnDisable()
	self:GetModule('MelodicVitals'):OnEnable()
	self:GetModule('MusicalEvents'):OnDisable()
	self:GetModule('MusicalEvents'):OnEnable()

	-- and finally delete the actual entry
	self.db.global.mediaDB[media] = nil
end
