local G = LibStub("AceGUI-3.0")
local L = LibStub('AceLocale-3.0'):GetLocale('ShivvAudio')

local LSM -- LibSharedMedia if available
local mainFrame -- outside ref to primary gui frame
local addonMediaPath = 'Interface\\AddOns\\ShivvAudio\\Media\\'
local mediaListDropdown = {}

local unitListDropdown = {
	['player'] = L.Player,
	['pet'] = L.Pet,
	['focus'] = L.Focus,
	['target'] = L.Target,
	['party1'] = L.Party1,
	['party2'] = L.Party2,
	['party3'] = L.Party3,
	['party4'] = L.Party4,
	['arena1'] = L.Arena1,
	['arena2'] = L.Arena2,
	['arena3'] = L.Arena3,
	['arena4'] = L.Arena4,
	['arena5'] = L.Arena5,
}

local function MakeItYellow(text) -- stupid but amuses me (and does mean less coding!)
	return '|cffffd100' .. text .. '|r'
end

local function ShivvTip(widget, title, text)
	GameTooltip:SetOwner(widget.frame, 'ANCHOR_TOPRIGHT')
	GameTooltip:SetText(title, 1, 1, 1, 1, 1)
	GameTooltip:AddLine(text, 1, 0.83, 0, 1, 1)
	GameTooltip:Show()
end

local function ZeroToNone(v)
	if (v == 0) then
		return 'none'
	else
		return v
	end
end

local function NoneToZero(v)
	if (v == 'none') then
		return 0
	else
		return v
	end
end

local function Spacer(w, h)
	local spacer = G:Create('Label')
	spacer:SetText('')
	spacer:SetWidth(w)
	spacer:SetHeight(h)
	return spacer
end


-- --------------- --
-- MAIN GUI WINDOW --
-- --------------- --
local function BuildTabTable()
	local tabTable = {
		[1] = {text = L.Main, value = 'main'},
		[2] = {text = L.MediaDatabase, value = 'media'}
	}

	if (ShivvAudio.db.profile.enableAudibleAuras) then
		tinsert(tabTable, {text = L.AudibleAuras, value = 'auras'})
	end

	if (ShivvAudio.db.profile.enableMelodicVitals) then
		tinsert(tabTable, {text = L.MelodicVitals, value = 'vitals'})
	end

	if (ShivvAudio.db.profile.enableMusicalEvents) then
		tinsert(tabTable, {text = L.MusicalEvents, value = 'events'})
	end

	if (ShivvAudio.db.profile.enableSymphoniousCombat) then
		tinsert(tabTable, {text = L.SymphoniousCombat, value = 'combat'})
	end

	return tabTable
end

function ShivvAudio:BuildGUI()
	if (self:IsMusicActive() or InCombatLockdown()) then -- don't open gui unless all 'musical' buffs are cancelled (avoids a lot of problems) and not in combat
		self:Print(L.NoGUIWhileMusicOrCombat)
		return
	end

	-- create a new AceGUI layout to avoid a bug with positioning two side-by-side-frames
	if (not G:GetLayout('KithTwoPanel')) then
		G:RegisterLayout('KithTwoPanel', function(content, children)
			if (children[1]) then
				local child = children[1]
				child.frame:ClearAllPoints()
				child.frame:Show()
				child.frame:SetPoint('TOPLEFT', content, 'TOPLEFT', 0, -5)
			end
			if (children[2]) then
				local child = children[2]
				child.frame:ClearAllPoints()
				child.frame:Show()
				child.frame:SetPoint('TOPRIGHT', content, 'TOPRIGHT', 0, -5)
			end
		end)
	end

	local f = G:Create('Frame')
	mainFrame = f -- outside ref
	f:SetTitle(L.Title .. '  |cffffffffv' .. GetAddOnMetadata('ShivvAudio', 'Version') .. '|r')
	f:SetStatusText(L['RandomStatus' .. tostring(random(1, 8))])
	f:SetCallback('OnClose', function(widget) G:Release(widget) end)
	f:SetLayout('Fill')
	f:Show()

	local t = G:Create('TabGroup')
	t:SetTitle('')
	t:SetTabs(BuildTabTable())
	t:SetLayout('Flow')
	t:SetCallback('OnGroupSelected', function(group, event, value)
		group:ReleaseChildren()

		if (value == 'media') then
			self:ShowMedia(group)
		elseif (value == 'auras') then
			self:ShowAudibleAuras(group)
		elseif (value == 'vitals') then
			self:ShowMelodicVitals(group)
		elseif (value == 'events') then
			self:ShowMusicalEvents(group)
		elseif (value == 'combat') then
			self:ShowSymphoniousCombat(group)
		else
			self:ShowMain(group)
		end
	end)
	t:SelectTab('main')
	f:AddChild(t)
end

-- ------------------- --
-- MEDIA LIST DROPDOWN --
-- ------------------- --
do
	local sortedList = {}
	local tsort = table.sort
	local mDB

	local function MakeItBlue(text)
		return '|cff40bad8' .. text .. '|r'
	end

	function ShivvAudio:PopulateMediaListDropdown()
		for k in pairs(mediaListDropdown) do mediaListDropdown[k] = nil end -- clear list

		mDB = self.db.global.mediaDB

		for k in pairs(mDB) do
			sortedList[#sortedList + 1] = k
		end
		tsort(sortedList)

		for i, key in ipairs(sortedList) do
			if (mDB[key].isMusic) then
				mediaListDropdown[key] = MakeItBlue(key)
			else
				mediaListDropdown[key] = key
			end
			sortedList[i] = nil
		end

		mediaListDropdown['none'] = L.None -- add 'none' entry
	end
end

-- ---------- --
-- MAIN PANEL --
-- ---------- --
local function BuildModuleToggle(f, mod)
	local t = G:Create('CheckBox')
	t:SetValue(ShivvAudio.db.profile['enable' .. mod])
	t:SetTriState(false)
	t:SetType('checkbox')
	t:SetLabel(MakeItYellow(L['Enable' .. mod]))
	t:SetDescription(L['EnableDesc' .. mod])
	t:SetCallback('OnValueChanged', function(w, e, v)
		ShivvAudio.db.profile['enable' .. mod] = v

		mainFrame:Hide()

		if (v) then
			ShivvAudio:GetModule(mod):OnEnable()
		else
			ShivvAudio:GetModule(mod):OnDisable()
		end

		ShivvAudio:BuildGUI()
	end)
	f:AddChild(t)
end

function ShivvAudio:ShowMain(f)
	local full = G:Create('SimpleGroup')
	full:SetLayout('KithTwoPanel')
	full:SetFullHeight(true)
	full:SetFullWidth(true)
	f:AddChild(full)

	local opt = G:Create('SimpleGroup')
	opt:SetLayout('List')
	opt:SetWidth(220)
	opt:SetFullHeight(true)
	full:AddChild(opt)

	BuildModuleToggle(opt, 'AudibleAuras')
	BuildModuleToggle(opt, 'MelodicVitals')
	BuildModuleToggle(opt, 'MusicalEvents')
	BuildModuleToggle(opt, 'SymphoniousCombat')

	local music = G:Create('CheckBox')
	music:SetValue(self.db.profile.gameMusicControl)
	music:SetTriState(false)
	music:SetType('checkbox')
	music:SetLabel(MakeItYellow(L.EnableMusicControl))
	music:SetDescription(L.EnableDescMusicControl)
	music:SetCallback('OnValueChanged', function(w, e, v)
		self.db.profile.gameMusicControl = v
	end)
	opt:AddChild(music)

	-- setup LSM import buton
	local lsm = G:Create('Button')
	lsm:SetWidth(200)

	LSM = LibStub:GetLibrary('LibSharedMedia-3.0', true)

	if (LSM) then -- LSM installed, activate button
		lsm:SetText(L.AddLSM)
		lsm:SetDisabled(false)
		lsm:SetCallback('OnClick', function(widget)
			self:LSMImport()
		end)
		lsm:SetCallback('OnEnter', function(widget)
			ShivvTip(widget, L.AddLSM, L.AddLSMDescEnabled)
		end)
		lsm:SetCallback('OnLeave', function(widget)
			GameTooltip:Hide()
		end)
	else -- no LSM installed, disable button
		lsm:SetText(L.AddLSMDisabled)
		lsm:SetDisabled(true)
	end
	opt:AddChild(lsm)

	local introBox = G:Create('InlineGroup')
	introBox:SetTitle(L.IntroTitle)
	introBox:SetWidth(400)
	introBox:SetLayout('Flow')
	full:AddChild(introBox)

	local introScroll = G:Create('ScrollFrame')
	introScroll:SetLayout('List')
	introScroll:SetRelativeWidth(1)
	introScroll:SetHeight(340)
	introBox:AddChild(introScroll)

	local intro = G:Create('Label')
	intro:SetText(L.Intro)
	intro:SetColor(1, 1, 1)
	intro:SetFontObject(GameFontNormalSmall)
	intro:SetRelativeWidth(1)
	introScroll:AddChild(intro)
end

-- -------------------- --
-- MEDIA DATABASE PANEL --
-- -------------------- --
do
	local sortedList = {}
	local tsort = table.sort
	local item, data, frame, scroll, addEdit
	local stopTimeout = 0

	local function IsItMusic(entry) -- colours musical tagged entries blue for reference
		if (entry) then
			return '|cff40bad8' .. L.Yes .. '|r'
		else
			return L.No
		end
	end

	local function PopulateMediaDatabase()
		-- clear data
		scroll:ReleaseChildren()
		scroll:SetUserData('selectedMedia', nil)
		scroll:SetUserData('selectedItemRef', nil)

		-- create an alphabetically sorted list of media
		for media in pairs(ShivvAudio.db.global.mediaDB) do
			sortedList[#sortedList + 1] = media
		end
		tsort(sortedList)

		-- create scrollframe entries for each media item
		for i, key in ipairs(sortedList) do
			data = ShivvAudio.db.global.mediaDB[key] -- local lookup reference
			item = G:Create('PentaTextInteractiveLabel')
			item:SetRelativeWidth(1)
			item:SetHighlight('Interface\\QuestFrame\\UI-QuestTitleHighlight')
			item:SetTextWidths(150, 60, 0, 0)
			item:SetColor()
			item:SetText(key, IsItMusic(data.isMusic), nil, nil, data.path)
			item:SetUserData('media', key)
			item:SetCallback('OnClick', function(w)
				if (w:IsSelected()) then -- already selected, deselect
					scroll:SetUserData('selectedMedia', nil)
					scroll:SetUserData('selecteItemRef', nil)
					ShivvAudio:ShowMediaAddEdit(true)
				else
					if (scroll:GetUserData('selectedItemRef') ~= nil) then -- one already selected, clear that before selecting self
						scroll:GetUserData('selectedItemRef'):UnlockHighlight()
					end

					scroll:SetUserData('selectedMedia', w:GetUserData('media'))
					scroll:SetUserData('selectedItemRef', w)
					ShivvAudio:ShowMediaAddEdit(false, w:GetUserData('media'))
				end
			end)
			scroll:AddChild(item)
			sortedList[i] = nil
		end
	end

	function ShivvAudio:ShowMedia(f)
		frame = f -- for later external ref

		local box = G:Create('InlineGroup')
		box:SetTitle(L.ListOfMedia)
		box:SetRelativeWidth(1)
		box:SetLayout('Flow')
		f:AddChild(box)

		local header = G:Create('PentaTextInteractiveLabel')
		header:SetRelativeWidth(1)
		header:SetHighlight(nil)
		header:SetColor(1, 0.83, 0, 1, 0.83, 0, nil, nil, nil, nil, nil, nil, 1, 0.83, 0)
		header:SetTextWidths(150, 60, 0, 0)
		header:SetText(L.MediaHeaderName, L.MediaHeaderMusic, nil, nil, L.MediaHeaderPath)
		header:SetFontObject(GameFontNormalSmall)
		box:AddChild(header)

		local s = G:Create('ScrollFrame')
		s:SetLayout('List')
		s:SetRelativeWidth(1)
		s:SetHeight(150)
		box:AddChild(s)

		scroll = s -- for later external ref

		PopulateMediaDatabase()

		local ae = G:Create('InlineGroup')
		ae:SetRelativeWidth(1)
		ae:SetLayout('Flow')
		f:AddChild(ae)
		addEdit = ae

		self:ShowMediaAddEdit(true)
	end

	function ShivvAudio:ShowMediaAddEdit(addMode, key)
		addEdit:ReleaseChildren()

		if (addMode) then
			addEdit:SetTitle(L.MediaAdd)
		else
			addEdit:SetTitle(L.MediaEdit)
		end

		local name = G:Create('EditBox')
		name:SetLabel(L.MediaName)
		name:SetWidth(250)
		if (not addMode) then
			name:SetText(key)
			name:SetDisabled(true)
		else
			name:SetDisabled(false)
		end
		name:SetCallback('OnEnter', function(w)
			ShivvTip(w, L.MediaNameTip, L.MediaNameDesc)
		end)
		name:SetCallback('OnLeave', function()
			GameTooltip:Hide()
		end)
		addEdit:AddChild(name)

		local music = G:Create('CheckBox')
		music:SetTriState(false)
		music:SetType('checkbox')
		music:SetWidth(100)
		music:SetLabel(L.MediaIsMusic)
		if (addMode) then
			music:SetValue(false)
		else
			music:SetValue(self.db.global.mediaDB[key].isMusic)
		end
		music:SetCallback('OnEnter', function(w)
			ShivvTip(w, L.MediaIsMusicTip, L.MediaIsMusicDesc)
		end)
		music:SetCallback('OnLeave', function()
			GameTooltip:Hide()
		end)
		addEdit:AddChild(music)

		local path = G:Create('EditBox')
		path:SetLabel(L.MediaPath)
		path:SetWidth(250)
		if (addMode) then
			path:SetCallback('OnEnter', function(w)
				ShivvTip(w, L.MediaPathTip, L.MediaPathDesc)
			end)
		else
			path:SetText(self.db.global.mediaDB[key].path)
			path:SetCallback('OnEnter', function(w)
				ShivvTip(w, L.MediaPathTip, L.MediaPathDescEdit)
			end)
		end
		path:SetCallback('OnLeave', function()
			GameTooltip:Hide()
		end)
		addEdit:AddChild(path)

		if (addMode) then
			local internal = G:Create('CheckBox')
			internal:SetTriState(false)
			internal:SetType('checkbox')
			internal:SetWidth(160)
			internal:SetLabel(L.MediaInShivv)
			internal:SetValue(false)
			internal:SetCallback('OnEnter', function(w)
				ShivvTip(w, L.MediaInShivvTip, L.MediaInShivvDesc)
			end)
			internal:SetCallback('OnLeave', function()
				GameTooltip:Hide()
			end)
			addEdit:AddChild(internal)

			local addIt = G:Create('Button')
			addIt:SetText(L.MediaAddIt)
			addIt:SetWidth(140)
			addIt:SetCallback('OnClick', function(w)
				if (name:GetText() == '' or path:GetText() == '' or self.db.global.mediaDB[name:GetText()] or self.internalMusicRef[name:GetText()]) then
					self:Print(L.MediaMustEnterUniqueNameAndPath)
				else
					local n, p, m, i = name:GetText(), path:GetText(), music:GetValue(), internal:GetValue()

					if (i) then p = addonMediaPath .. p end -- prepend shivvaudio\media path if is internal sound

					-- create new media entry
					self.db.global.mediaDB[n] = {
						['isMusic'] = m,
						['path'] = p,
					}

					PopulateMediaDatabase(self)
					self:Print(L.NewMediaAdded)
					self:ShowMediaAddEdit(true)
				end
			end)
			addEdit:AddChild(addIt)
		else
			local sg = G:Create('SimpleGroup')
			sg:SetLayout('Flow')
			sg:SetRelativeWidth(1)
			addEdit:AddChild(sg)

			local editIt = G:Create('Button')
			editIt:SetText(L.MediaEditIt)
			editIt:SetWidth(140)
			editIt:SetCallback('OnClick', function(w)
				if (path:GetText() == '') then -- checks for a path entry
					self:Print(L.MediaMustEnterPath)
					path:SetText(self.db.global.mediaDB[key].path)
				else
					local p, m = path:GetText(), music:GetValue()

					-- update entry
					if (self.db.global.mediaDB[key]) then
						self.db.global.mediaDB[key].isMusic = m
						self.db.global.mediaDB[key].path = p
					end

					PopulateMediaDatabase(self)
					self:Print(L.EditedMedia)
					self:ShowMediaAddEdit(true) -- hide edit box as we've deselected the edited option
				end
			end)
			addEdit:AddChild(editIt)

			local delIt = G:Create('Button')
			delIt:SetText(L.MediaDelIt)
			delIt:SetWidth(140)
			delIt:SetCallback('OnClick', function(w)
				self:RemoveMedia(key)

				PopulateMediaDatabase(self)
				self:Print(L.DeletedMedia)
				self:ShowMediaAddEdit(true)
			end)
			addEdit:AddChild(delIt)

			local play = G:Create('Button')
			play:SetText(L.PlayMedia)
			play:SetWidth(140)
			play:SetCallback('OnClick', function(w)
				self:SoundHandler(key)
				self:Print(L.PlayingMedia)
			end)
			addEdit:AddChild(play)

			local stop = G:Create('Button')
			stop:SetText(L.StopMedia)
			stop:SetWidth(140)
			stop:SetCallback('OnClick', function(w)
				if (GetTime() > stopTimeout + 5) then
					self:Print(L.StoppedPlaying)
					stopTimeout = GetTime()
					Sound_GameSystem_RestartSoundSystem()
				else
					self:Print(L.CannotStopPlaying)
				end
			end)
			addEdit:AddChild(stop)
		end
	end
end

-- ------------------- --
-- AUDIBLE AURAS PANEL --
-- ------------------- --
do
	local unitList, buffList, interimList = {}, {}, {}
	local tsort = table.sort
	local item, frame, scroll, data, addEdit
	local sortedBy = 'unit'

	local function ShowNiceValue(v)
		if (v == 0) then
			return '--'
		else
			return v
		end
	end

	local function sizeOf(tbl)
		local n = 0
		for _ in pairs(tbl) do
			n = n + 1
		end

		return n
	end

	local function AddAuraDatabaseItem(unit, buff, onGain, onLoss, music)
		item = G:Create('PentaTextInteractiveLabel')
		item:SetRelativeWidth(1)
		item:SetHighlight('Interface\\QuestFrame\\UI-QuestTitleHighlight')
		item:SetTextWidths(70, 170, 120, 120)
		item:SetColor()
		item:SetText(unit, buff, ShowNiceValue(onGain), ShowNiceValue(onLoss), ShowNiceValue(music))
		item:SetUserData('unit', unit)
		item:SetUserData('buff', buff)
		item:SetCallback('OnClick', function(w)
			if (w:IsSelected()) then
				scroll:SetUserData('selectedUnit', nil)
				scroll:SetUserData('selectedBuff', nil)
				scroll:SetUserData('selectedItemRef', nil)
				ShivvAudio:ShowAuraAddEdit(true)
			else
				if (scroll:GetUserData('selectedItemRef') ~= nil) then
					scroll:GetUserData('selectedItemRef'):UnlockHighlight()
				end

				scroll:SetUserData('selectedUnit', w:GetUserData('unit'))
				scroll:SetUserData('selectedBuff', w:GetUserData('buff'))
				scroll:SetUserData('selectedItemRef', w)
				ShivvAudio:ShowAuraAddEdit(false, w:GetUserData('unit'), w:GetUserData('buff'))
			end
		end)
		scroll:AddChild(item)
	end

	local function PopulateAuraDatabase()
		scroll:ReleaseChildren()
		scroll:SetUserData('selectedUnit', nil)
		scroll:SetUserData('selectedBuff', nil)
		scroll:SetUserData('selectedItemRef', nil)

		if (sortedBy == 'unit') then
			for unit in pairs(ShivvAudio.db.profile.audibleAuras) do
				unitList[#unitList + 1] = unit
			end
			tsort(unitList)

			for i, unit in ipairs(unitList) do
				for buff in pairs(ShivvAudio.db.profile.audibleAuras[unit]) do
					buffList[#buffList + 1] = buff
				end
				tsort(buffList)

				for j, buff in ipairs(buffList) do
					data = ShivvAudio.db.profile.audibleAuras[unit][buff]

					AddAuraDatabaseItem(unit, buff, data.onGain, data.onLoss, data.music)

					buffList[j] = nil
				end

				unitList[i] = nil
			end
		else
			-- create a 'new' auraDB that section buf buff first, then unit
			for unit in pairs(ShivvAudio.db.profile.audibleAuras) do
				for buff in pairs(ShivvAudio.db.profile.audibleAuras[unit]) do
					if (not interimList[buff]) then
						interimList[buff] = {}
					end

					interimList[buff][unit] = true
				end
			end

			for buff in pairs(interimList) do
				buffList[#buffList + 1] = buff
			end
			tsort(buffList)

			for i, buff in ipairs(buffList) do
				for unit in pairs(interimList[buff]) do
					unitList[#unitList + 1] = unit
				end
				tsort(unitList)

				for j, unit in ipairs(unitList) do
					data = ShivvAudio.db.profile.audibleAuras[unit][buff]
					AddAuraDatabaseItem(unit, buff, data.onGain, data.onLoss, data.music)
					unitList[j] = nil
				end
				buffList[i] = nil
			end

			-- clean out interim table
			for buff in pairs(interimList) do
				for unit in pairs(interimList[buff]) do
					interimList[buff][unit] = nil
				end
				interimList[buff] = nil
			end
		end
	end

	function ShivvAudio:ShowAudibleAuras(f)
		frame = f

		local box = G:Create('InlineGroup')
		box:SetTitle(L.ListOfAuras)
		box:SetRelativeWidth(1)
		box:SetLayout('Flow')
		f:AddChild(box)

		local header = G:Create('PentaTextInteractiveLabel')
		header:SetRelativeWidth(1)
		header:SetHighlight(nil)
		header:SetColor(1, 0.83, 0, 1, 0.83, 0, 1, 0.83, 0, 1, 0.83, 0, 1, 0.83, 0)
		header:SetTextWidths(70, 170, 120, 120)
		header:SetText(L.AuraHeaderUnit, L.AuraHeaderBuff, L.AuraHeaderGain, L.AuraHeaderLoss, L.AuraHeaderMusic)
		header:SetFontObject(GameFontNormalSmall)
		box:AddChild(header)

		local s = G:Create('ScrollFrame')
		s:SetLayout('List')
		s:SetRelativeWidth(1)
		s:SetHeight(150)
		box:AddChild(s)

		scroll = s

		PopulateAuraDatabase(self)

		local sort = G:Create('Button')
		sort:SetWidth(140)
		sort:SetText(L.SortListByBuff)
		sort:SetCallback('OnClick', function(w)
			if (sortedBy == 'unit') then
				sortedBy = 'buff'
				w:SetText(L.SortListByUnit)
			else
				sortedBy = 'unit'
				w:SetText(L.SortListByBuff)
			end
			PopulateAuraDatabase(self)
			self:ShowAuraAddEdit(true)
		end)
		f:AddChild(sort)

		local ae = G:Create('InlineGroup')
		ae:SetRelativeWidth(1)
		ae:SetLayout('Flow')
		f:AddChild(ae)
		addEdit = ae

		self:ShowAuraAddEdit(true)
	end

	function ShivvAudio:ShowAuraAddEdit(addMode, unit, buff)
		addEdit:ReleaseChildren()

		self:PopulateMediaListDropdown()

		if (addMode) then
			addEdit:SetTitle(L.AuraAdd)
		else
			addEdit:SetTitle(L.AuraEdit)
		end

		local m = G:Create('Dropdown') -- created here as referenced by following section

		local uBox = G:Create('SimpleGroup')
		uBox:SetLayout('List')
		uBox:SetWidth(140)
		addEdit:AddChild(uBox)

		local uLabel = G:Create('Label')
		uLabel:SetText(L.AuraUnit)
		uBox:AddChild(uLabel)

		local u = G:Create('Dropdown')
		u:SetMultiselect(false)
		u:SetWidth(130)
		u:SetList(unitListDropdown)
		if (addMode) then
			u:SetValue('player')
		else
			u:SetValue(unit)
		end
		u:SetDisabled(not addMode)
		u:SetCallback('OnEnter', function(w)
			ShivvTip(w, L.AuraUnitTip, L.AuraUnitDesc)
		end)
		u:SetCallback('OnLeave', function(w)
			GameTooltip:Hide()
		end)
		u:SetCallback('OnValueChanged', function(w, e, v)
			if (v == 'player') then
				m:SetDisabled(false)
			else
				m:SetDisabled(true)
			end
		end)
		uBox:AddChild(u)

		local onGBox = G:Create('SimpleGroup')
		onGBox:SetLayout('List')
		onGBox:SetWidth(140)
		addEdit:AddChild(onGBox)

		local uLabel = G:Create('Label')
		uLabel:SetText(L.AuraOnGain)
		onGBox:AddChild(uLabel)

		local onG = G:Create('Dropdown')
		onG:SetMultiselect(false)
		onG:SetWidth(130)
		onG:SetList(mediaListDropdown)
		if (addMode) then
			onG:SetValue('none')
		else
			onG:SetValue(ZeroToNone(self.db.profile.audibleAuras[unit][buff].onGain))
		end
		onG:SetCallback('OnEnter', function(w)
			ShivvTip(w, L.AuraOnGainTip, L.AuraOnGainDesc)
		end)
		onG:SetCallback('OnLeave', function(w)
			GameTooltip:Hide()
		end)
		onGBox:AddChild(onG)

		local onLBox = G:Create('SimpleGroup')
		onLBox:SetLayout('List')
		onLBox:SetWidth(140)
		addEdit:AddChild(onLBox)

		local uLabel = G:Create('Label')
		uLabel:SetText(L.AuraOnLoss)
		onLBox:AddChild(uLabel)

		local onL = G:Create('Dropdown')
		onL:SetMultiselect(false)
		onL:SetWidth(130)
		onL:SetList(mediaListDropdown)
		if (addMode) then
			onL:SetValue('none')
		else
			onL:SetValue(ZeroToNone(self.db.profile.audibleAuras[unit][buff].onLoss))
		end
		onL:SetCallback('OnEnter', function(w)
			ShivvTip(w, L.AuraOnLossTip, L.AuraOnLossDesc)
		end)
		onL:SetCallback('OnLeave', function(w)
			GameTooltip:Hide()
		end)
		onLBox:AddChild(onL)

		local mBox = G:Create('SimpleGroup')
		mBox:SetLayout('List')
		mBox:SetWidth(140)
		addEdit:AddChild(mBox)

		local uLabel = G:Create('Label')
		uLabel:SetText(L.AuraMusic)
		mBox:AddChild(uLabel)

		-- this item is created above
		m:SetMultiselect(false)
		m:SetWidth(130)
		m:SetList(mediaListDropdown)
		if (addMode) then
			m:SetValue('none')
		else
			m:SetValue(ZeroToNone(self.db.profile.audibleAuras[unit][buff].music))

			if (unit ~= 'player') then
				m:SetDisabled(true)
			else
				m:SetDisabled(false)
			end
		end
		m:SetCallback('OnEnter', function(w)
			ShivvTip(w, L.AuraMusicTip, L.AuraMusicDesc)
		end)
		m:SetCallback('OnLeave', function(w)
			GameTooltip:Hide()
		end)
		mBox:AddChild(m)

		local b = G:Create('EditBox')
		b:SetLabel(L.AuraBuff)
		b:SetWidth(220)
		if (not addMode) then
			b:SetText(buff)
		end
		b:SetDisabled(not addMode)
		b:SetCallback('OnEnter', function(w)
			ShivvTip(w, L.AuraBuffTip, L.AuraBuffDesc)
		end)
		b:SetCallback('OnLeave', function(w)
			GameTooltip:Hide()
		end)
		addEdit:AddChild(b)

		if (addMode) then
			addEdit:AddChild(Spacer(212, 5))

			local ae = G:Create('Button')
			ae:SetWidth(150)
			ae:SetText(L.AuraAddButton)
			ae:SetCallback('OnClick', function(w)
				if (b:GetText() == '') then
					self:Print(L.AuraEntryMustEnterAura)
				elseif (self.db.profile.audibleAuras[u:GetValue()] and self.db.profile.audibleAuras[u:GetValue()][b:GetText()]) then
					self:Print(L.AuraEntryAlreadyExists)
				else
					-- create a new unit entry if needed
					if (not self.db.profile.audibleAuras[u:GetValue()]) then -- no unit entry yet, make one
						self.db.profile.audibleAuras[u:GetValue()] = {}
					end

					-- create a new aura entry
					self.db.profile.audibleAuras[u:GetValue()][b:GetText()] = {
						['onGain'] = NoneToZero(onG:GetValue()),
						['onLoss'] = NoneToZero(onL:GetValue()),
						['music'] = 0,
					}

					-- configure music entry based on whether unit is a player
					if (u:GetValue() == 'player' and m:GetValue() ~= 'none') then
						self.db.profile.audibleAuras[u:GetValue()][b:GetText()].music = m:GetValue()
					end

					-- got all the values and made an entry... repopulate list and clear fields
					PopulateAuraDatabase(self)

					self:ShowAuraAddEdit(true)

					-- and setup audible auras
					self:GetModule('AudibleAuras'):OnDisable()
					self:GetModule('AudibleAuras'):OnEnable()
				end
			end)
			addEdit:AddChild(ae)
		else
			addEdit:AddChild(Spacer(52, 5))

			local del = G:Create('Button')
			del:SetWidth(150)
			del:SetText(L.AuraDelButton)
			del:SetCallback('OnClick', function(w)
				-- delete the item, clean out the unit entry if its the last buff
				if (self.db.profile.audibleAuras[unit] and self.db.profile.audibleAuras[unit][buff]) then -- just check they exist
					self.db.profile.audibleAuras[unit][buff] = nil

					if (sizeOf(self.db.profile.audibleAuras[unit]) == 0) then -- this was the last entry for this unit, clear it out
						self.db.profile.audibleAuras[unit] = nil
					end

					self:Print(L.AuraEntryDeleted)
					PopulateAuraDatabase(self)
					self:ShowAuraAddEdit(true)
					self:GetModule('AudibleAuras'):OnDisable()
					self:GetModule('AudibleAuras'):OnEnable()
				end
			end)
			addEdit:AddChild(del)

			addEdit:AddChild(Spacer(10, 5))

			local ae = G:Create('Button')
			ae:SetWidth(150)
			ae:SetText(L.AuraEditButton)
			ae:SetCallback('OnClick', function(w)
				if (self.db.profile.audibleAuras[unit] and self.db.profile.audibleAuras[unit][buff]) then -- just check they exist
					-- can only edit gain/loss/music entries, so only check them

					self.db.profile.audibleAuras[unit][buff].onGain = NoneToZero(onG:GetValue())
					self.db.profile.audibleAuras[unit][buff].onLoss = NoneToZero(onL:GetValue())

					if (unit == 'player') then -- is player so set music as well
						self.db.profile.audibleAuras[unit][buff].music = NoneToZero(m:GetValue())
					end
				end

				self:Print(L.AuraEntryEdited)
				PopulateAuraDatabase(self)
				self:ShowAuraAddEdit(true)
			end)

			addEdit:AddChild(ae)
		end
	end
end

-- -------------------- --
-- MELODIC VITALS PANEL --
-- -------------------- --
do
	local tabTable = {
		[1] = {text = L.Player, value = 'player'},
		[2] = {text = L.Pet, value = 'pet'},
		[3] = {text = L.Focus, value = 'focus'},
		[4] = {text = L.Target, value = 'target'},
		[5] = {text = L.Party1, value = 'party1'},
		[6] = {text = L.Party2, value = 'party2'},
		[7] = {text = L.Party3, value = 'party3'},
		[8] = {text = L.Party4, value = 'party4'},
		[9] = {text = L.Arena1, value = 'arena1'},
		[10] = {text = L.Arena2, value = 'arena2'},
		[11] = {text = L.Arena3, value = 'arena3'},
		[12] = {text = L.Arena4, value = 'arena4'},
		[13] = {text = L.Arena5, value = 'arena5'}
	}

	local function BuildVitalOption(f, unit, db)
		local hdb = ShivvAudio.db.profile.melodicVitals[db]

		local h = G:Create('InlineGroup')
		h:SetRelativeWidth(1)
		if (db == 'healthDB') then
			h:SetTitle(L.SetupHealthAlert)
		else
			h:SetTitle(L.SetupPowerAlert)
		end
		h:SetLayout('Flow')
		f:AddChild(h)

		local hs = G:Create('Dropdown')
		local hp = G:Create('Slider')
		local he = G:Create('CheckBox')

		local hsBox = G:Create('SimpleGroup')
		hsBox:SetLayout('List')
		hsBox:SetWidth(150)
		h:AddChild(hsBox)

		local hsLabel = G:Create('Label')
		hsLabel:SetText(L.Sound)
		hsBox:AddChild(hsLabel)

		hs:SetMultiselect(false)
		hs:SetWidth(130)
		hs:SetList(mediaListDropdown)
		hs:SetCallback('OnValueChanged', function(w, e, v)
			if (v == 'none') then
				ShivvAudio.db.profile.melodicVitals[db][unit].sound = 0
			else
				ShivvAudio.db.profile.melodicVitals[db][unit].sound = v
			end
   		end)
		hs:SetValue(hdb[unit] and ZeroToNone(hdb[unit].sound) or 'none')
		if (hdb[unit] and hdb[unit].enable) then
			hs:SetDisabled(false)
		else
			hs:SetDisabled(true)
		end
		hsBox:AddChild(hs)

		hp:SetSliderValues(0.05, 0.95, 0.05)
		hp:SetIsPercent(true)
		if (db == 'healthDB') then
			hp:SetLabel(L.HealthLevel)
		else
			hp:SetLabel(L.PowerLevel)
		end
		hp:SetWidth(180)
		hp:SetCallback('OnValueChanged', function(w, e, v)
			-- round value out to nearest 0.05 (5%) multiple and save
			ShivvAudio.db.profile.melodicVitals[db][unit].percent = (math.floor(math.floor((v * 100) + 0.5) / 5) * 5) / 100
		end)
		hp:SetValue(hdb[unit] and hdb[unit].percent or 0.5)
		if (hdb[unit] and hdb[unit].enable) then
			hp:SetDisabled(false)
		else
			hp:SetDisabled(true)
		end
		h:AddChild(hp)

		h:AddChild(Spacer(40, 5))

		he:SetTriState(false)
		he:SetType('checkbox')
		he:SetLabel(MakeItYellow(L.Enable))
		he:SetDescription('')
		he:SetWidth(140)
		he:SetCallback('OnEnter', function(w)
			ShivvTip(w, L.EnableTip, L.EnableDesc)
		end)
		he:SetCallback('OnLeave', function(w)
			GameTooltip:Hide()
		end)
		he:SetCallback('OnValueChanged', function(w, e, v)
			if (hdb[unit]) then
				ShivvAudio.db.profile.melodicVitals[db][unit].enable = v
			else
				-- only called on first enable
				ShivvAudio.db.profile.melodicVitals[db][unit] = {
					['enable'] = true,
					['percent'] = 0.5,
					['sound'] = 0,
				}
			end

			hs:SetDisabled(not v)
			hp:SetDisabled(not v)

			ShivvAudio:GetModule('MelodicVitals'):OnDisable()
			ShivvAudio:GetModule('MelodicVitals'):OnEnable()
		end)
		he:SetValue(hdb[unit] and hdb[unit].enable or false)
		h:AddChild(he)
	end

	function ShivvAudio:ShowMelodicVitals(f)
		self:PopulateMediaListDropdown()

		local i = G:Create('Label')
		i:SetText(L.VitalsIntro)
		i:SetRelativeWidth(0.55)
		f:AddChild(i)

		local rt = G:Create('Slider')
		rt:SetSliderValues(1, 20, 1)
		rt:SetIsPercent(false)
		rt:SetLabel(L.RepeatTimer)
		rt:SetWidth(200)
		rt:SetCallback('OnEnter', function(w)
			ShivvTip(w, L.RepeatTimerTip, L.RepeatTimerDesc)
		end)
		rt:SetCallback('OnLeave', function(w)
			GameTooltip:Hide()
		end)
		rt:SetCallback('OnValueChanged', function(w, e, v)
			ShivvAudio.db.profile.melodicVitals.repeatTimer = v

			self:GetModule('MelodicVitals'):VitalRepeatSetupHP()
			self:GetModule('MelodicVitals'):VitalRepeatSetupPWR()
		end)
		rt:SetValue(ShivvAudio.db.profile.melodicVitals.repeatTimer)
		f:AddChild(rt)

		local tab = G:Create('TabGroup')
		tab:SetTitle('')
		tab:SetTabs(tabTable)
		tab:SetLayout('Flow')
		tab:SetFullWidth(true)
		tab:SetHeight(200)
		tab:SetCallback('OnGroupSelected', function(g, e, v)
			g:ReleaseChildren()

			BuildVitalOption(g, v, 'healthDB')
			BuildVitalOption(g, v, 'powerDB')
		end)
		tab:SelectTab('player')
		f:AddChild(tab)
	end
end

-- -------------------- --
-- MUSICAL EVENTS PANEL --
-- -------------------- --
do

	local function BuildEventPanel(f, event)
		local box = G:Create('SimpleGroup')
		box:SetLayout('KithTwoPanel')
		box:SetRelativeWidth(0.95)
		box:SetHeight(50)
		f:AddChild(box)

		local label = G:Create('Label')
		label:SetText('\n' .. MakeItYellow(L[event]))
		label:SetFontObject(GameFontNormalSmall)
		box:AddChild(label)

		local sound = G:Create('Dropdown')
		sound:SetMultiselect(false)
		sound:SetWidth(130)
		sound:SetList(mediaListDropdown)
		sound:SetValue(ZeroToNone(ShivvAudio.db.profile.musicalEvents[event]))
		sound:SetCallback('OnValueChanged', function(w, e, v)
			ShivvAudio.db.profile.musicalEvents[event] = NoneToZero(v)
			ShivvAudio:PopulateShivvInternalMusicRef()
			ShivvAudio:GetModule('MusicalEvents'):OnDisable()
			ShivvAudio:GetModule('MusicalEvents'):OnEnable()
		end)
		box:AddChild(sound)
	end

	function ShivvAudio:ShowMusicalEvents(f)
		self:PopulateMediaListDropdown()

		local full = G:Create('SimpleGroup')
		full:SetFullHeight(true)
		full:SetFullWidth(true)
		full:SetLayout('KithTwoPanel')
		f:AddChild(full)

		local sEvent = G:Create('SimpleGroup')
		sEvent:SetRelativeWidth(0.45)
		sEvent:SetFullHeight(true)
		full:AddChild(sEvent)

		local sEventBlurb = G:Create('Label')
		sEventBlurb:SetText(L.SEvent)
		sEventBlurb:SetRelativeWidth(1)
		sEvent:AddChild(sEventBlurb)

		local mEvent = G:Create('SimpleGroup')
		mEvent:SetRelativeWidth(0.45)
		mEvent:SetFullHeight(true)
		full:AddChild(mEvent)

		local mEventBlurb = G:Create('Label')
		mEventBlurb:SetText(L.MEvent)
		mEventBlurb:SetRelativeWidth(1)
		mEvent:AddChild(mEventBlurb)

		BuildEventPanel(sEvent, 'achievement')
		BuildEventPanel(sEvent, 'death')
		BuildEventPanel(sEvent, 'levelup')
		BuildEventPanel(sEvent, 'lootopen')
		BuildEventPanel(sEvent, 'lootclose')
		BuildEventPanel(mEvent, 'auction')
		BuildEventPanel(mEvent, 'bank')
		BuildEventPanel(mEvent, 'crafting')
		BuildEventPanel(mEvent, 'looting')
		BuildEventPanel(mEvent, 'shop')
	end
end

-- ------------------------ --
-- SYMPHONIOUS COMBAT PANEL --
-- ------------------------ --
do
	local tabTable = {
		[1] = {text = L.Main, value = 'main'},
		[2] = {text = L.Outside, value = 'outside'},
		[3] = {text = L.Party, value = 'party'},
		[4] = {text = L.Raid, value = 'raid'},
		[5] = {text = L.Battleground, value = 'battleground'},
	}

	local function BuildAreaToggle(f, area)
		local t = G:Create('CheckBox')
		t:SetValue(ShivvAudio.db.profile.symphoniousCombat['enable' .. area])
		t:SetTriState(false)
		t:SetType('checkbox')
		t:SetLabel(MakeItYellow(L['Enable' .. area]))
		t:SetDescription(L['EnableDesc' .. area])
		t:SetCallback('OnValueChanged', function(w, e, v)
			ShivvAudio.db.profile.symphoniousCombat['enable' .. area] = v
			ShivvAudio:GetModule('SymphoniousCombat'):OnDisable()
			ShivvAudio:GetModule('SymphoniousCombat'):OnEnable()
		end)
		f:AddChild(t)
	end

	local function SetTierLabel(s1, s2)
		local slider1 = ShivvAudio.db.profile.symphoniousCombat[s1]
		local slider2 = ShivvAudio.db.profile.symphoniousCombat[s2]

		if (slider1 == 1) then
			if (slider2 == 2) then
				return format('|cffffd100%s|r     %s\n|cffffd100%s|r    %d %s\n|cffffd100%s|r   %d%s', L.TierI, L.OneMob, L.TierII, 2, L.Mobs, L.TierIII, slider2 + 1, L.PlusMobs)
			else
				return format('|cffffd100%s|r     %s\n|cffffd100%s|r    %d-%d %s\n|cffffd100%s|r   %d%s', L.TierI, L.OneMob, L.TierII, slider1 + 1, slider2, L.Mobs, L.TierIII, slider2 + 1, L.PlusMobs)
			end
		else
			if ((slider2 - slider1) == 1) then
				return format('|cffffd100%s|r     %d-%d %s\n|cffffd100%s|r    %d %s\n|cffffd100%s|r   %d%s', L.TierI, 1, slider1, L.Mobs, L.TierII, slider2, L.Mobs, L.TierIII, slider2 + 1, L.PlusMobs)
			else
					return format('|cffffd100%s|r     %d-%d %s\n|cffffd100%s|r    %d-%d %s\n|cffffd100%s|r   %d%s', L.TierI, 1, slider1, L.Mobs, L.TierII, slider1 + 1, slider2, L.Mobs, L.TierIII, slider2 + 1, L.PlusMobs)
			end
		end
	end

	local function BuildDropdown(f, label, audio)
		local b = G:Create('SimpleGroup')
		b:SetLayout('KithTwoPanel')
		b:SetRelativeWidth(0.95)
		b:SetHeight(50)
		f:AddChild(b)

		local bL = G:Create('Label')
		bL:SetText('\n' .. MakeItYellow(label))
		bL:SetFontObject(GameFontNormalSmall)
		b:AddChild(bL)

		local bD = G:Create('Dropdown')
		bD:SetMultiselect(false)
		bD:SetWidth(130)
		bD:SetList(mediaListDropdown)
		bD:SetValue(ZeroToNone(ShivvAudio.db.profile.symphoniousCombat.audio[audio]))
		bD:SetCallback('OnValueChanged', function(w, e, v)
			ShivvAudio.db.profile.symphoniousCombat.audio[audio] = NoneToZero(v)
			ShivvAudio:PopulateShivvInternalMusicRef()
		end)
		b:AddChild(bD)
	end

	function ShivvAudio:ShowSymphoniousCombat(f)
		self:PopulateMediaListDropdown()

		local tab = G:Create('TabGroup')
		tab:SetTitle('')
		tab:SetTabs(tabTable)
		tab:SetLayout('Flow')
		tab:SetFullWidth(true)
		tab:SetFullHeight(true)
		tab:SetCallback('OnGroupSelected', function(g, e, v)
			g:ReleaseChildren()

			if (v == 'main') then
				self:ShowSymphoniousCombatMain(g)
			elseif (v == 'outside') then
				self:ShowSymphoniousCombatOutside(g)
			elseif (v == 'party') then
				self:ShowSymphoniousCombatParty(g)
			elseif (v == 'raid') then
				self:ShowSymphoniousCombatRaid(g)
			elseif (v == 'battleground') then
				self:ShowSymphoniousCombatBattleground(g)
			end
		end)
		tab:SelectTab('main')
		f:AddChild(tab)
	end

	function ShivvAudio:ShowSymphoniousCombatMain(f)
		local full = G:Create('SimpleGroup')
		full:SetLayout('KithTwoPanel')
		full:SetFullHeight(true)
		full:SetFullWidth(true)
		f:AddChild(full)

		local opt = G:Create('SimpleGroup')
		opt:SetLayout('List')
		opt:SetWidth(220)
		opt:SetFullHeight(true)
		full:AddChild(opt)

		BuildAreaToggle(opt, 'Outside')
		BuildAreaToggle(opt, 'Party')
		BuildAreaToggle(opt, 'Raid')
		BuildAreaToggle(opt, 'Battleground')

		opt:AddChild(Spacer(20, 20))

		local t = G:Create('CheckBox')
		t:SetValue(ShivvAudio.db.profile.symphoniousCombat.advanced)
		t:SetTriState(false)
		t:SetType('checkbox')
		t:SetLabel(MakeItYellow(L.EnableAdvanced))
		t:SetDescription(L.EnableDescAdvanced)
		t:SetCallback('OnValueChanged', function(w, e, v)
			ShivvAudio.db.profile.symphoniousCombat.advanced = v
			ShivvAudio:GetModule('SymphoniousCombat'):OnDisable()
			ShivvAudio:GetModule('SymphoniousCombat'):OnEnable()
		end)
		opt:AddChild(t)

		local introBox = G:Create('InlineGroup')
		introBox:SetTitle(L.CombatIntroTitle)
		introBox:SetWidth(400)
		introBox:SetLayout('Flow')
		full:AddChild(introBox)

		local introScroll = G:Create('ScrollFrame')
		introScroll:SetLayout('List')
		introScroll:SetRelativeWidth(1)
		introScroll:SetHeight(290)
		introBox:AddChild(introScroll)

		local intro = G:Create('Label')
		intro:SetText(L.CombatIntro)
		intro:SetColor(1, 1, 1)
		intro:SetFontObject(GameFontNormalSmall)
		intro:SetRelativeWidth(1)
		introScroll:AddChild(intro)
	end

	function ShivvAudio:ShowSymphoniousCombatOutside(f)
		local full = G:Create('SimpleGroup')
		full:SetLayout('KithTwoPanel')
		full:SetFullHeight(true)
		full:SetFullWidth(true)
		f:AddChild(full)

		local left = G:Create('SimpleGroup')
		left:SetLayout('List')
		left:SetRelativeWidth(0.45)
		left:SetFullHeight(true)
		full:AddChild(left)

		local right = G:Create('SimpleGroup')
		right:SetLayout('List')
		right:SetRelativeWidth(0.45)
		right:SetFullHeight(true)
		full:AddChild(right)

		BuildDropdown(left, L.OnEnteringCombat, 'outsideenter')
		BuildDropdown(left, L.OnLeavingCombat, 'outsideleave')

		if (not ShivvAudio.db.profile.symphoniousCombat.advanced) then
			BuildDropdown(right, L.WhileInCombat, 'outside1')
		else -- advanced mode
			BuildDropdown(right, L.WhenInTierI, 'outside1')
			BuildDropdown(right, L.WhenInTierII, 'outside2')
			BuildDropdown(right, L.WhenInTierIII, 'outside3')
			BuildDropdown(right, L.WhenFightingElite, 'outsideelite')

			local tiers = G:Create('Label')
			tiers:SetText(SetTierLabel('outsideMaxT1', 'outsideMaxT2'))
			tiers:SetFontObject(GameFontNormal)
			right:AddChild(tiers)

			local t1 = G:Create('Slider')
			local t2 = G:Create('Slider')

			right:AddChild(Spacer(20, 14))

			t1:SetIsPercent(false)
			t1:SetSliderValues(1, 10, 1)
			t1:SetWidth(180)
			t1:SetLabel(L.TierIMaxMobs)
			t1:SetValue(ShivvAudio.db.profile.symphoniousCombat.outsideMaxT1)
			t1:SetCallback('OnValueChanged', function(w, e, v)
				ShivvAudio.db.profile.symphoniousCombat.outsideMaxT1 = v

				if (ShivvAudio.db.profile.symphoniousCombat.outsideMaxT2 < (v + 1)) then
					-- t2 max is to low, bump up to minimum possible
					ShivvAudio.db.profile.symphoniousCombat.outsideMaxT2 = v + 1
					t2:SetValue(ShivvAudio.db.profile.symphoniousCombat.outsideMaxT2)
				elseif (ShivvAudio.db.profile.symphoniousCombat.outsideMaxT2 > (v + 11)) then
					-- to high a t2 max, lower it to new max
					ShivvAudio.db.profile.symphoniousCombat.outsideMaxT2 = v + 11
					t2:SetValue(ShivvAudio.db.profile.symphoniousCombat.outsideMaxT2)
				end

				t2:SetSliderValues(v + 1, v + 11)
				tiers:SetText(SetTierLabel('outsideMaxT1', 'outsideMaxT2'))
			end)
			right:AddChild(t1)

			t2:SetIsPercent(false)
			t2:SetSliderValues(ShivvAudio.db.profile.symphoniousCombat.outsideMaxT1 + 1, ShivvAudio.db.profile.symphoniousCombat.outsideMaxT1 + 11, 1)
			t2:SetWidth(180)
			t2:SetLabel(L.TierIIMaxMobs)
			t2:SetValue(ShivvAudio.db.profile.symphoniousCombat.outsideMaxT2)
			t2:SetCallback('OnValueChanged', function(w, e, v)
				ShivvAudio.db.profile.symphoniousCombat.outsideMaxT2 = v
				tiers:SetText(SetTierLabel('outsideMaxT1', 'outsideMaxT2'))
			end)
			right:AddChild(t2)
		end
	end

	function ShivvAudio:ShowSymphoniousCombatParty(f)
		local full = G:Create('SimpleGroup')
		full:SetLayout('KithTwoPanel')
		full:SetFullHeight(true)
		full:SetFullWidth(true)
		f:AddChild(full)

		local left = G:Create('SimpleGroup')
		left:SetLayout('List')
		left:SetRelativeWidth(0.45)
		left:SetFullHeight(true)
		full:AddChild(left)

		local right = G:Create('SimpleGroup')
		right:SetLayout('List')
		right:SetRelativeWidth(0.45)
		right:SetFullHeight(true)
		full:AddChild(right)

		BuildDropdown(left, L.OnEnteringCombat, 'partyenter')
		BuildDropdown(left, L.OnLeavingCombat, 'partyleave')

		if (not ShivvAudio.db.profile.symphoniousCombat.advanced) then
			BuildDropdown(right, L.WhileInCombat, 'party1')
		else -- advanced mode
			BuildDropdown(right, L.WhenInTierI, 'party1')
			BuildDropdown(right, L.WhenInTierII, 'party2')
			BuildDropdown(right, L.WhenInTierIII, 'party3')
			BuildDropdown(right, L.WhenFightingBoss, 'partyboss')

			local tiers = G:Create('Label')
			tiers:SetText(SetTierLabel('partyMaxT1', 'partyMaxT2'))
			tiers:SetFontObject(GameFontNormal)
			right:AddChild(tiers)

			local t1 = G:Create('Slider')
			local t2 = G:Create('Slider')

			right:AddChild(Spacer(20, 14))

			t1:SetIsPercent(false)
			t1:SetSliderValues(1, 10, 1)
			t1:SetWidth(180)
			t1:SetLabel(L.TierIMaxMobs)
			t1:SetValue(ShivvAudio.db.profile.symphoniousCombat.partyMaxT1)
			t1:SetCallback('OnValueChanged', function(w, e, v)
				ShivvAudio.db.profile.symphoniousCombat.partyMaxT1 = v

				if (ShivvAudio.db.profile.symphoniousCombat.partyMaxT2 < (v + 1)) then
					-- t2 max is to low, bump up to minimum possible
					ShivvAudio.db.profile.symphoniousCombat.partyMaxT2 = v + 1
					t2:SetValue(ShivvAudio.db.profile.symphoniousCombat.partyMaxT2)
				elseif (ShivvAudio.db.profile.symphoniousCombat.partyMaxT2 > (v + 11)) then
					-- to high a t2 max, lower it to new max
					ShivvAudio.db.profile.symphoniousCombat.partyMaxT2 = v + 11
					t2:SetValue(ShivvAudio.db.profile.symphoniousCombat.partyMaxT2)
				end

				t2:SetSliderValues(v + 1, v + 11)
				tiers:SetText(SetTierLabel('partyMaxT1', 'partyMaxT2'))
			end)
			right:AddChild(t1)

			t2:SetIsPercent(false)
			t2:SetSliderValues(ShivvAudio.db.profile.symphoniousCombat.partyMaxT1 + 1, ShivvAudio.db.profile.symphoniousCombat.partyMaxT1 + 11, 1)
			t2:SetWidth(180)
			t2:SetLabel(L.TierIIMaxMobs)
			t2:SetValue(ShivvAudio.db.profile.symphoniousCombat.partyMaxT2)
			t2:SetCallback('OnValueChanged', function(w, e, v)
				ShivvAudio.db.profile.symphoniousCombat.partyMaxT2 = v
				tiers:SetText(SetTierLabel('partyMaxT1', 'partyMaxT2'))
			end)
			right:AddChild(t2)
		end
	end

	function ShivvAudio:ShowSymphoniousCombatRaid(f)
		local full = G:Create('SimpleGroup')
		full:SetLayout('KithTwoPanel')
		full:SetFullHeight(true)
		full:SetFullWidth(true)
		f:AddChild(full)

		local left = G:Create('SimpleGroup')
		left:SetLayout('List')
		left:SetRelativeWidth(0.45)
		left:SetFullHeight(true)
		full:AddChild(left)

		local right = G:Create('SimpleGroup')
		right:SetLayout('List')
		right:SetRelativeWidth(0.45)
		right:SetFullHeight(true)
		full:AddChild(right)

		BuildDropdown(left, L.OnEnteringCombat, 'raidenter')
		BuildDropdown(left, L.OnLeavingCombat, 'raidleave')

		if (not ShivvAudio.db.profile.symphoniousCombat.advanced) then
			BuildDropdown(right, L.WhileInCombat, 'raidtrash')
		else -- advanced mode
			BuildDropdown(right, L.WhenFightingTrash, 'raidtrash')
			BuildDropdown(right, L.WhenFightingBoss, 'raidboss')
		end
	end

	function ShivvAudio:ShowSymphoniousCombatBattleground(f)
		local full = G:Create('SimpleGroup')
		full:SetLayout('KithTwoPanel')
		full:SetFullHeight(true)
		full:SetFullWidth(true)
		f:AddChild(full)

		local left = G:Create('SimpleGroup')
		left:SetLayout('List')
		left:SetRelativeWidth(0.45)
		left:SetFullHeight(true)
		full:AddChild(left)

		local right = G:Create('SimpleGroup')
		right:SetLayout('List')
		right:SetRelativeWidth(0.45)
		right:SetFullHeight(true)
		full:AddChild(right)

		BuildDropdown(left, L.OnEnteringCombat, 'battlegroundenter')
		BuildDropdown(left, L.OnLeavingCombat, 'battlegroundleave')
		BuildDropdown(right, L.WhileInCombat, 'battleground')
	end
end

-- -------------- --
-- MISC FUNCTIONS --
-- -------------- --
function ShivvAudio:LSMImport()
	local path
	local entriesAdded = 0

	local media = LSM:List('sound')

	for k, name in pairs(media) do
		path = LSM:Fetch('sound', name)

		if (path == '' or path == nil or path == 'None' or path == 'none' or path == 0 or self.db.global.mediaDB[name]) then
			-- do nothing if bad path, or if the entry already exists
		else
			self.db.global.mediaDB[name] = {
				['isMusic'] = false,
				['path'] = path,
			}
			entriesAdded = entriesAdded + 1
		end
	end

	self:Print(entriesAdded .. L.LSMEntriesAdded)
end
